(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@nativescript/core'), require('@imagene.me/nativescript-shadow')) :
    typeof define === 'function' && define.amd ? define('nativescript-shadow-angular', ['exports', '@angular/core', '@nativescript/core', '@imagene.me/nativescript-shadow'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global["nativescript-shadow-angular"] = {}, global.ng.core, global["ns-core"], global["ns-nativescript-shadow"]));
})(this, (function (exports, core$1, core, nativescriptShadow) { 'use strict';

    var NativeShadowDirective = /** @class */ (function () {
        function NativeShadowDirective(el, render) {
            var _this = this;
            this.el = el;
            this.render = render;
            this.loaded = false;
            this.initialized = false;
            this.eventsBound = false;
            this.monkeyPatch = function (val) {
                _this.previousNSFn.call(_this.el.nativeElement, val);
                _this.applyShadow();
            };
            if (core.isAndroid) {
                this.originalNSFn = this.el.nativeElement._redrawNativeBackground; //always store the original method
            }
        }
        NativeShadowDirective.prototype.ngOnInit = function () {
            if (!this.initialized) {
                this.initialized = true;
                this.initializeCommonData();
                if (core.isAndroid) {
                    this.initializeAndroidData();
                }
                else if (core.isIOS) {
                    this.initializeIOSData();
                }
                if (this.shadow && this.shadow.elevation) {
                    if (core.isAndroid) {
                        this.loadFromAndroidData(this.shadow);
                    }
                    else if (core.isIOS) {
                        this.loadFromIOSData(this.shadow);
                    }
                }
                this.bindEvents();
            }
        };
        NativeShadowDirective.prototype.ngOnDestroy = function () {
            if (this.initialized) {
                this.unbindEvents();
                this.initialized = false;
            }
        };
        // NS ListViews create elements dynamically
        // loaded and unloaded are called before angular is "ready"
        // https://github.com/NativeScript/nativescript-angular/issues/1221#issuecomment-422813111
        // So we ensure we're running loaded/unloaded events outside of angular
        NativeShadowDirective.prototype.bindEvents = function () {
            if (!this.eventsBound) {
                core.addWeakEventListener(this.el.nativeElement, core.View.loadedEvent, this.onLoaded, this);
                core.addWeakEventListener(this.el.nativeElement, core.View.unloadedEvent, this.onUnloaded, this);
                this.eventsBound = true;
                // in some cases, the element is already loaded by time of binding
                if (this.el.nativeElement.isLoaded) {
                    this.onLoaded();
                }
            }
        };
        NativeShadowDirective.prototype.unbindEvents = function () {
            if (this.eventsBound) {
                core.removeWeakEventListener(this.el.nativeElement, core.View.loadedEvent, this.onLoaded, this);
                core.removeWeakEventListener(this.el.nativeElement, core.View.unloadedEvent, this.onUnloaded, this);
                this.eventsBound = false;
            }
        };
        NativeShadowDirective.prototype.onLoaded = function () {
            this.loaded = true;
            // Weirdly ngOnInit isn't called on iOS on demo app
            // Managed to get it working on iOS when applying to
            // FlexboxLayout, but on the demo app, we apply to a
            // Label, and, for that case, ngOnInit isn't called
            // This is just enforcing the Directive is initialized
            // before calling this.applyShadow()
            if (!this.initialized) {
                this.ngOnInit();
            }
            this.applyShadow();
            if (core.isAndroid) {
                this.previousNSFn = this.el.nativeElement._redrawNativeBackground; // just to maintain compatibility with other patches
                this.el.nativeElement._redrawNativeBackground = this.monkeyPatch;
            }
        };
        NativeShadowDirective.prototype.addIosWrapper = function () {
            var _a;
            if (core.isIOS) {
                var originalElement = this.el.nativeElement;
                this.iosShadowRapper = this.render.createElement('GridLayout');
                this.cloneViewPositionProperties(originalElement, this.iosShadowRapper);
                // wrappingElement.cssClasses = mainElement.cssClasses;
                var parent = originalElement.parentNode;
                this.render.insertBefore(parent, this.iosShadowRapper, originalElement);
                this.render.removeChild(parent, originalElement);
                if (originalElement.parentNode) {
                    (_a = originalElement.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(originalElement);
                }
                this.render.appendChild(this.iosShadowRapper, originalElement);
            }
        };
        NativeShadowDirective.prototype.onUnloaded = function () {
            this.loaded = false;
            if (core.isAndroid) {
                this.el.nativeElement._redrawNativeBackground = this.originalNSFn; // always revert to the original method
            }
        };
        NativeShadowDirective.prototype.ngAfterViewInit = function () {
            this.addIosWrapper();
        };
        NativeShadowDirective.prototype.ngOnChanges = function (changes) {
            if (this.loaded &&
                !!changes &&
                (changes.hasOwnProperty('shadow') ||
                    changes.hasOwnProperty('elevation') ||
                    changes.hasOwnProperty('pressedElevation') ||
                    changes.hasOwnProperty('shape') ||
                    changes.hasOwnProperty('bgcolor') ||
                    changes.hasOwnProperty('cornerRadius') ||
                    changes.hasOwnProperty('pressedTranslationZ') ||
                    changes.hasOwnProperty('forcePressAnimation') ||
                    changes.hasOwnProperty('translationZ') ||
                    changes.hasOwnProperty('maskToBounds') ||
                    changes.hasOwnProperty('shadowColor') ||
                    changes.hasOwnProperty('shadowOffset') ||
                    changes.hasOwnProperty('shadowOpacity') ||
                    changes.hasOwnProperty('shadowRadius') ||
                    changes.hasOwnProperty('rasterize') ||
                    changes.hasOwnProperty('useShadowMap'))) {
                if (changes.hasOwnProperty('shadow') &&
                    !changes.hasOwnProperty('elevation') &&
                    typeof changes.shadow.currentValue === 'number') {
                    this.elevation = changes.shadow.currentValue;
                }
                if (changes.shadow && changes.shadow.currentValue.elevation) {
                    if (core.isAndroid) {
                        this.loadFromAndroidData(this.shadow);
                    }
                    else if (core.isIOS) {
                        this.loadFromIOSData(this.shadow);
                    }
                }
                this.applyShadow();
            }
        };
        NativeShadowDirective.prototype.applyShadow = function () {
            if (this.shadow === null ||
                this.shadow === undefined ||
                (this.shadow === '' && !this.elevation)) {
                return;
            }
            // For shadows to be shown on Android the SDK has to be greater
            // or equal than 21, lower SDK means no setElevation method is available
            if (core.isAndroid) {
                if (android.os.Build.VERSION.SDK_INT < 21) {
                    return;
                }
            }
            var viewToApplyShadowTo = core.isIOS
                ? this.iosShadowRapper
                : this.el.nativeElement;
            if (viewToApplyShadowTo) {
                nativescriptShadow.Shadow.apply(viewToApplyShadowTo, {
                    elevation: this.elevation,
                    pressedElevation: this.pressedElevation,
                    shape: this.shape,
                    bgcolor: this.bgcolor,
                    cornerRadius: this.cornerRadius,
                    translationZ: this.translationZ,
                    pressedTranslationZ: this.pressedTranslationZ,
                    forcePressAnimation: this.forcePressAnimation,
                    maskToBounds: this.maskToBounds,
                    shadowColor: this.shadowColor,
                    shadowOffset: this.shadowOffset,
                    shadowOpacity: this.shadowOpacity,
                    shadowRadius: this.shadowRadius,
                    rasterize: this.rasterize,
                    useShadowPath: this.useShadowPath
                });
            }
        };
        NativeShadowDirective.prototype.initializeCommonData = function () {
            var tShadow = typeof this.shadow;
            if ((tShadow === 'string' || tShadow === 'number') && !this.elevation) {
                this.elevation = this.shadow ? parseInt(this.shadow, 10) : 2;
            }
            var tElevation = typeof this.elevation;
            if (tElevation === 'string' || tElevation === 'number') {
                this.elevation = this.elevation
                    ? parseInt(this.elevation, 10)
                    : 2;
            }
        };
        NativeShadowDirective.prototype.initializeAndroidData = function () {
            if (typeof this.cornerRadius === 'string') {
                this.cornerRadius = parseInt(this.cornerRadius, 10);
            }
            if (typeof this.translationZ === 'string') {
                this.translationZ = parseInt(this.translationZ, 10);
            }
        };
        NativeShadowDirective.prototype.initializeIOSData = function () {
            if (typeof this.shadowOffset === 'string') {
                this.shadowOffset = parseFloat(this.shadowOffset);
            }
            if (typeof this.shadowOpacity === 'string') {
                this.shadowOpacity = parseFloat(this.shadowOpacity);
            }
            if (typeof this.shadowRadius === 'string') {
                this.shadowRadius = parseFloat(this.shadowRadius);
            }
        };
        NativeShadowDirective.prototype.loadFromAndroidData = function (data) {
            this.elevation = data.elevation || this.elevation;
            this.shape = data.shape || this.shape;
            this.bgcolor = data.bgcolor || this.bgcolor;
            this.cornerRadius = data.cornerRadius || this.cornerRadius;
            this.translationZ = data.translationZ || this.translationZ;
        };
        NativeShadowDirective.prototype.loadFromIOSData = function (data) {
            this.maskToBounds = data.maskToBounds || this.maskToBounds;
            this.shadowColor = data.shadowColor || this.shadowColor;
            this.shadowOffset = data.shadowOffset || this.shadowOffset;
            this.shadowOpacity = data.shadowOpacity || this.shadowOpacity;
            this.shadowRadius = data.shadowRadius || this.shadowRadius;
            this.rasterize = data.rasterize || this.rasterize;
            this.useShadowPath = data.useShadowPath || this.useShadowPath;
        };
        NativeShadowDirective.prototype.cloneViewPositionProperties = function (sourceView, targetView) {
            if (sourceView.row > 0) {
                targetView.row = sourceView.row;
            }
            if (sourceView.rowSpan > 0) {
                targetView.rowSpan = sourceView.rowSpan;
            }
            if (sourceView.col > 0) {
                targetView.col = sourceView.col;
            }
            if (sourceView.colSpan > 0) {
                targetView.colSpan = sourceView.colSpan;
            }
            if (sourceView.verticalAlignment) {
                targetView.verticalAlignment = sourceView.verticalAlignment;
            }
            if (sourceView.horizontalAlignment) {
                targetView.horizontalAlignment = sourceView.horizontalAlignment;
            }
        };
        return NativeShadowDirective;
    }());
    NativeShadowDirective.decorators = [
        { type: core$1.Directive, args: [{ selector: '[shadow]' },] }
    ];
    NativeShadowDirective.ctorParameters = function () { return [
        { type: core$1.ElementRef },
        { type: core$1.Renderer2 }
    ]; };
    NativeShadowDirective.propDecorators = {
        shadow: [{ type: core$1.Input }],
        elevation: [{ type: core$1.Input }],
        pressedElevation: [{ type: core$1.Input }],
        shape: [{ type: core$1.Input }],
        bgcolor: [{ type: core$1.Input }],
        cornerRadius: [{ type: core$1.Input }],
        translationZ: [{ type: core$1.Input }],
        pressedTranslationZ: [{ type: core$1.Input }],
        forcePressAnimation: [{ type: core$1.Input }],
        maskToBounds: [{ type: core$1.Input }],
        shadowColor: [{ type: core$1.Input }],
        shadowOffset: [{ type: core$1.Input }],
        shadowOpacity: [{ type: core$1.Input }],
        shadowRadius: [{ type: core$1.Input }],
        useShadowPath: [{ type: core$1.Input }],
        rasterize: [{ type: core$1.Input }]
    };

    var NgShadowModule = /** @class */ (function () {
        function NgShadowModule() {
        }
        return NgShadowModule;
    }());
    NgShadowModule.decorators = [
        { type: core$1.NgModule, args: [{
                    imports: [],
                    declarations: [NativeShadowDirective],
                    exports: [NativeShadowDirective],
                    providers: [],
                },] }
    ];
    // Uncomment this line if the package provides a custom view component
    // registerElement('NativescriptShadow', () => NativescriptShadow);

    /**
     * Generated bundle index. Do not edit.
     */

    exports.NativeShadowDirective = NativeShadowDirective;
    exports.NgShadowModule = NgShadowModule;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=nativescript-shadow-angular.umd.js.map
